package meerkat.registry.AsyncRegistryCallbacks;

import com.google.common.util.concurrent.FutureCallback;
import com.google.protobuf.InvalidProtocolBufferException;
import javassist.NotFoundException;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.VoterRegistry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

import static meerkat.util.BulletinBoardUtils.getLatestMessage;


/**
 * Created by Vladimir Eliezer  on 3/12/2016.
 * AsyncRegistry getVoter creates this callback to get voterInfo protobuff
 */
public class GetVoterCallback implements FutureCallback<List<BulletinBoardMessage>> {
    protected FutureCallback<VoterRegistry.VoterInfo> callback;
    final Logger logger = LoggerFactory.getLogger(getClass());

    public GetVoterCallback(FutureCallback<VoterRegistry.VoterInfo> callback){
        logger.debug("Initialized GetVoterCallback : ", new java.util.Date().getTime());
        this.callback = callback;
    }

    @Override
    public void onSuccess(List<BulletinBoardMessage> messages) {
        if(messages.isEmpty()){
            callback.onFailure(new NotFoundException("Voter info not found in database."));
            return;
        }
        BulletinBoardMessage latestMessage = getLatestMessage(messages);

        try {
            VoterRegistry.VoterInfo info = VoterRegistry.VoterInfo.newBuilder()
                    .mergeFrom(latestMessage.getMsg().getData()).build();

            logger.debug("Wanted voter information: ", info.getId().getId(), info.getInfo(),
                         new java.util.Date().getTime());

                    callback.onSuccess(info);
        } catch (InvalidProtocolBufferException e) {
            logger.error("Error occurred while trying to get voter information: ",
                    new java.util.Date().getTime(),
                    e);
            callback.onFailure(e);
        }
    }

    @Override
    public void onFailure(Throwable t) {
        logger.error("Error occurred while trying to get voter information:",
                new java.util.Date().getTime(),
                t);
        callback.onFailure(t);
    }
}
